import sys,os
import gedit
import gtk

def select_tree(ix,mm,*ax):
  ix=ix[0]
  if ix<0 or ix>=len(mm.marqv): return False
  doc=mm.window.get_active_document()
  doc.goto_line(mm.marqv[ix][0])
  mm.window.get_active_view().scroll_to_cursor()
  return True

###############################################################################
# Window instance

class MM:

  def __init__(self,plugin,window):
    self.plugin=plugin
    self.window=window
    try:
      panel=self.window.get_side_panel()
      assert panel is not None
    except: return
    
    self.treestore=gtk.TreeStore(str)
    self.treeview=gtk.TreeView(self.treestore)
    col=gtk.TreeViewColumn("Name")
    self.treeview.append_column(col)
    renderer=gtk.CellRendererText()
    col.pack_start(renderer,True)
    col.add_attribute(renderer,"text",0)
    sel=self.treeview.get_selection()
    sel.set_select_function(select_tree,self)
    swin=gtk.ScrolledWindow()
    swin.add_with_viewport(self.treeview)
    swin.show()
    self.treeview.show()
    
    try:
      icon=gtk.Image()
      icon.set_from_file(os.path.join(sys.path[0],"marqueemark/mm-icon.png"))
    except: icon=gtk.Image()
    panel.add_item(swin,"Marquee Mark",icon)
    self.marqv=[] # (lineno,title)
    
  def deactivate(self):
    pass
    
  def update_ui(self):
    doc=self.window.get_active_document()
    if doc is None:
      return
    src=doc.get_text(doc.get_start_iter(),doc.get_end_iter())
    srcp=0
    lineno=1
    gotmarq=False
    self.wipe_marquees()
    while srcp<len(src):
      nl=0
      while (srcp+nl<len(src)) and (src[srcp+nl]!='\n') and (src[srcp+nl]!='\r'): nl+=1
      if srcp+nl>=len(src): break
      line=src[srcp:srcp+nl].strip()
      srcp+=nl+1
      if gotmarq: # previous line was a marquee start; this line is its title
        if line.startswith("# ") or line.startswith("* "): line=line[2:].strip()
        self.add_marquee(lineno-1,line)
        gotmarq=False
      else: # is this line a marquee start?
        multiple=False
        while len(line)>1 and line[-1]==line[-2]: 
          line=line[:-1]
          multiple=True
        if multiple and (line=='#' or line=='/*'): gotmarq=True
      lineno+=1
      
  def wipe_marquees(self):
    self.marqv=[]
    self.treestore.clear()
    
  def add_marquee(self,lineno,title):
    self.marqv.append((lineno,title))
    self.treestore.append(None,["%d: %s"%(lineno,title)])

###############################################################################
# The Plug-in

class MarqeeMark(gedit.Plugin):

  def __init__(self):
    gedit.Plugin.__init__(self)
    self.windowd={}
    
  def activate(self,window):
    self.windowd[window]=MM(self,window)
    
  def deactivate(self,window):
    self.windowd[window].deactivate()
    del self.windowd[window]
    
  def update_ui(self,window):
    self.windowd[window].update_ui()
